
# -*- coding: utf-8 -*-

"""dataManagement.py
   Author: Jorge Cardenas

   1. Simulation data logging in CSV Files
   2. Simulation data Retrieval

   Future developments:
   1. Local or remote data storage
"""

import csv
import os
import pandas as pd
import os.path as path

import subprocess

class Builder:

    def __init__(self, ansysPath,modelName,projectName, designName, modelPath,scriptPath,exportPath, imagesPath):
        self.ansysPath = ansysPath
        self.modelName = modelName
        self.modelPath = modelPath
        self.exportPath = exportPath
        self.projectName = projectName
        self.designName = designName
        self.imagesPath = imagesPath
        self.scriptPath = scriptPath

        isExist = os.path.exists(self.exportPath)
        if not isExist:

            # Create a new directory because it does not exist
            os.makedirs(self.exportPath)
            print("The new Exports directory is created!")

        isExist = os.path.exists(self.imagesPath)
        
        if not isExist:

            # Create a new directory because it does not exist
            os.makedirs(self.imagesPath)
            print("The new Images directory is created!")

    def create(self):
        pathString=self.scriptPath + self.modelName+".py"
        subprocess.run([self.ansysPath,"-RunScriptandExit",pathString])

    def simulate(self, filepath):
        subprocess.run([self.ansysPath,"-RunScriptandExit",filepath])

    def sim_file(self, parameters, batch, iteration, filePath, **kwargs):
    
        reports=kwargs['reports']
        simulationID=kwargs['simulation_id']
        variableName=kwargs['variable_name']
        value=kwargs['value']
        units=kwargs['units']


        tag = "_"+str(batch)+"-"+str(iteration)

        os.chdir( os.path.normpath(filePath))
        drawing = '"' + self.modelPath + self.projectName + '.aedt"'
        print(drawing)

        isExist = os.path.exists(self.exportPath +"/output/"+str(simulationID)+"/files/")


        if not isExist:

            # Create a new directory because it does not exist
            os.makedirs(self.exportPath +"/output/"+str(simulationID)+"/files/")
            print("The new directory is created!")

        isExist2 = os.path.exists(self.imagesPath +"/"+str(simulationID)+"/")


        if not isExist2:

            # Create a new directory because it does not exist
            os.makedirs(self.imagesPath +"/"+str(simulationID)+"/")
            print("The new directory is created!")


        f = open("intermediateFile.py", "w")  

        #f.write("\n")

        f.write("import ScriptEnv\n")
        f.write("import sys\n")
        f.write("import os\n")
        f.write("sys.path.insert(0, './src/')\n")
        f.write("from dbuilder.modules import hfss \n")


        f.write("oDesktop.RestoreWindow()\n")
        f.write("oDesktop.OpenProject(" + drawing + ")\n")
        f.write("oProject = oDesktop.SetActiveProject(" + '"'+self.projectName+'"' + ")\n")
        f.write('hfss.setVariable(oProject,"' + variableName + ' ","' + value + units + '","' +self.designName +'")\n')
        
        f.write('oDesign = oProject.SetActiveDesign("' +self.designName  + '")\n')
        f.write("oDesign.AnalyzeAll()")
        f.write("\n")
        f.write('oModule = oDesign.GetModule("OutputVariable")\n')

        for key, val in reports.items():
            f.write('oModule.CreateOutputVariable("' + str(key) + '","'+str(val)+'", "Setup1 : Sweep", "Modal Solution Data", [])\n')

        
        for key, val in reports.items():

            f.write('hfss.createResult(oProject,"' + self.exportPath +'","'+ tag +'","'+ self.designName  +'","'+ simulationID  +'","'+str(key)+'")\n')
        

        
        fileName= str(self.modelName+"_"+ simulationID+tag ) 
        f.write('hfss.exportImage(oProject,"' + self.imagesPath+"/"+str(simulationID) +'","'+fileName+'")\n')


        

        f.close()

    def sim_file_eigen(self, parameters, batch, iteration, filePath, **kwargs):
    
        reports=kwargs['reports']
        simulationID=kwargs['simulation_id']
        variableName=kwargs['variable_name']
        value=kwargs['value']
        units=kwargs['units']
        simfile=kwargs['simfile']

        tag = "_"+str(batch)+"_"+str(iteration)

        os.chdir( os.path.normpath(filePath))
        drawing = '"' + self.modelPath + self.projectName + '.aedt"'
        print(drawing)

        isExist = os.path.exists(self.exportPath +"/output/"+str(simulationID)+"/files/")


        if not isExist:

            # Create a new directory because it does not exist
            os.makedirs(self.exportPath +"/output/"+str(simulationID)+"/files/")
            print("The new directory is created!")

        f = open(simfile, "w")  

        #f.write("\n")

        f.write("import ScriptEnv\n")
        f.write("import sys\n")
        f.write("import os\n")
        f.write("sys.path.insert(0, './src/')\n")
        f.write("from dbuilder.modules import hfss \n")


        f.write("oDesktop.RestoreWindow()\n")
        f.write("oDesktop.OpenProject(" + drawing + ")\n")
        f.write("oProject = oDesktop.SetActiveProject(" + '"'+self.projectName+'"' + ")\n")

        for idx,item in enumerate(value): 
            f.write('hfss.setVariable(oProject,"' + variableName[idx] + ' ","' + item + units + '","' +self.designName +'")\n')
        
        f.write('oDesign = oProject.SetActiveDesign("' +self.designName  + '")\n')
        f.write("oDesign.AnalyzeAll()")
        f.write("\n")

        
        #for key, val in reports.items():
        #    f.write('oModule.CreateOutputVariable("' + str(key) + '","'+str(val)+'", "Setup1 : Sweep", "Modal Solution Data", [])\n')

        
        for key, val in reports.items():

            f.write('hfss.createEigenReport(oProject,"' + self.exportPath +'","'+ tag +'","'+ self.designName  +'","'+ simulationID  +'","'+str(key)+'")\n')
        

        f.close()



class DataExplorer:
    pass


   
class DBManager:

   file=None
   df = None
   def __init__(self, ansysPath,modelName,projectName, designName, modelPath,dbPath,dbName ):
        self.ansysPath = ansysPath
        self.modelName = modelName
        self.modelPath = modelPath
        self.dbPath = dbPath
        self.projectName = projectName
        self.designName = designName 
        self.dbName = dbName 

        isExist = os.path.exists(self.dbPath)
        if not isExist:

            # Create a new directory because it does not exist
            os.makedirs(self.dbPath)
            print("The new DB directory is created!")

        
   def load_df(self, columns):

      if path.exists(self.dbPath+self.dbName):
         self.df = pd.read_csv(self.dbPath+self.dbName, header=0)
      else:
            
        column_names = columns
        self.df = pd.DataFrame(columns = column_names.keys())

        for  (key, value) in columns.items():
            self.df[key]=self.df[key].astype( value)
 
   def insert_row(self,data_struct):
   
      df = pd.concat([self.df, pd.DataFrame([data_struct])])
      
      df.to_csv(self.dbPath+self.dbName, index=False,sep=',')
      
  
      
     





# ----------------------------------------------
# Script Recorded by Ansys Electronics Desktop Version 2021.2.0
# 13:55:07  Dec 02, 2023
# ----------------------------------------------
import ScriptEnv
ScriptEnv.Initialize("Ansoft.ElectronicsDesktop")
oDesktop.RestoreWindow()
oProject = oDesktop.NewProject()
oProject.InsertDesign("HFSS", "HFSSDesign1", "HFSS Modal Network", "")
oDesign = oProject.SetActiveDesign("HFSSDesign1")
oDesign.SetSolutionType("Eigenmode", 
	[
		"NAME:Options",
		"EnableAutoOpen:="	, False
	])
oDesign.SetSolutionType("HFSS Modal Network")
oDesign.SetSolutionType("Eigenmode", 
	[
		"NAME:Options",
		"EnableAutoOpen:="	, False
	])
oEditor = oDesign.SetActiveEditor("3D Modeler")
oEditor.Insert3DComponent(
	[
		"NAME:InsertComponentData",
		"TargetCS:="		, "Global",
		"ComponentFile:="	, "C:/Users/jorge/Documents/Projects Jorge C/Glide_Symmetry/Designs/Models/GlideCell2.a3dcomp",
		"IsLocal:="		, False,
		"UniqueIdentifier:="	, "",
		[
			"NAME:InstanceParameters",
			"GeometryParameters:="	, "gap=\'2mm\' gWidth=\'1.75mm\' h1_factor=\'0.29999999999999999\' h2_factor=\'0.40000000000000002\' offset_factor=\'0.10000000000000001\' Period=\'8mm\' sep_factor=\'0.5\' SubsH=\'0.76200000000000001mm\' w1_factor=\'0.20000000000000001\' w2_factor=\'0.125\'",
			"MaterialParameters:="	, "",
			"DesignParameters:="	, ""
		]
	])
oEditor = oDesign.SetActiveEditor("3D Modeler")
oEditor.Delete(
	[
		"NAME:Selections",
		"Selections:="		, "cell1"
	])



# import opencv
import cv2
import os
import numpy as np
import imutils  # https://pypi.org/project/imutils/


def cropImage(fullpath,image_path, image_name,output_path, resize_dim):
    # Read an image
    # input_image = cv2.imread(path, cv2.IMREAD_UNCHANGED)
    input_image = cv2.imread(fullpath, cv2.IMREAD_UNCHANGED)


    # Convert from BGR to HSV color space
    hsv = cv2.cvtColor(input_image, cv2.COLOR_BGR2HSV)

    # Get the saturation plane - all black/white/gray pixels are zero, and colored pixels are above zero.
    s = hsv[:, :, 1]

    #cv2.imshow('image', s) 
    # Apply threshold on s - use automatic threshold algorithm (use THRESH_OTSU).
    _, thresh = cv2.threshold(s, 10, 255, cv2.THRESH_BINARY+cv2.THRESH_OTSU)

    # Find contours
    cnts = cv2.findContours(thresh, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_NONE)
    cnts = imutils.grab_contours(cnts) 

    # Find the contour with the maximum area.
    c = max(cnts, key=cv2.contourArea)

    # Get bounding rectangle
    x, y, w, h = cv2.boundingRect(c)

    # Crop the bounding rectangle out of img
    output_image = input_image[y:y+h, x:x+w, :].copy()

    dim = resize_dim
  
    # resize image
    output_image = cv2.resize(output_image, dim, interpolation = cv2.INTER_AREA)  

    # Save image
    isExist = os.path.exists(output_path +"/processed/")



    
    if not isExist:

        # Create a new directory because it does not exist
        os.makedirs(output_path +"/processed/")
        print("The new directory is created!")
    cv2.imwrite(output_path +"/processed/"+image_name, output_image)
    cv2.waitKey(0)
    cv2.destroyAllWindows()

    return output_image


# -*- coding: utf-8 -*-
"""
@author: Jorge Cardenas
"""

def createProject():
    
    import ScriptEnv
    oDesktop.RestoreWindow()
    oProject = oDesktop.NewProject()
    

def hffsDesing(oProject, name):
    
    oProject.InsertDesign("HFSS", name, "HFSS Modal Network", "")

def solutionType(oProject, name, solution_type):
    #solution_type =Eigenmode
	#"HFSS Modal Network"

    oDesign = oProject.SetActiveDesign(name)
    oDesign.SetSolutionType(solution_type, [
		"NAME:Options",
		"EnableAutoOpen:="	, False
	])

def load3DObject(oDesign, componentFilePath):
    #component file path
	#  "C:/Users/jorge/Documents/Projects Jorge C/Glide_Symmetry/Designs/Models/GlideCell2.a3dcomp""C:/Users/jorge/Documents/Projects Jorge C/Glide_Symmetry/Designs/Models/GlideCell2.a3dcomp"
    oEditor = oDesign.SetActiveEditor("3D Modeler")
    oEditor.Insert3DComponent(
	[
		"NAME:InsertComponentData",
		"TargetCS:="		, "Global",
		"ComponentFile:="	, componentFilePath,
		"IsLocal:="		, False,
		"UniqueIdentifier:="	, "",
		[
			"NAME:InstanceParameters",
			"GeometryParameters:="	, "gap=\'2mm\' gWidth=\'1.75mm\' h1_factor=\'0.29999999999999999\' h2_factor=\'0.40000000000000002\' offset_factor=\'0.10000000000000001\' Period=\'8mm\' sep_factor=\'0.5\' SubsH=\'0.76200000000000001mm\' w1_factor=\'0.20000000000000001\' w2_factor=\'0.125\'",
			"MaterialParameters:="	, "",
			"DesignParameters:="	, ""
		]
	])
    
def setVariable(proj,name,value, design):
    oDesign = proj.SetActiveDesign(design)
    oDesign.ChangeProperty(
    	[
    		"NAME:AllTabs",
    		[
    			"NAME:LocalVariableTab",
    			[
    				"NAME:PropServers", 
    				"LocalVariables"
    			],
    			[
    				"NAME:ChangedProps",
    				[
    					"NAME:"+ name,
    					"Value:="		, value
    				]
    			]
    		]
    	])

    
def exportImage(proj,imagesPath,imageName):
	
	oDesign = proj.SetActiveDesign("HFSSDesign1")
	oEditor = oDesign.SetActiveEditor("3D Modeler")
	oEditor.ExportModelImageToFile(imagesPath+"/"+imageName+".png", 1156, 634, 
	[
		"NAME:SaveImageParams",
		"ShowAxis:="		, "False",
		"ShowGrid:="		, "False",
		"ShowRuler:="		, "False",
		"ShowRegion:="		, "Default",
		"Selections:="		, "",
		"FieldPlotSelections:="	, "",
		"Orientation:="		, "Top"
	])


def createResult(proj,path,name,design,simID, resultName):

    oDesign = proj.SetActiveDesign(design)
    oModule = oDesign.GetModule("ReportSetup")
    oModule.CreateReport(resultName, "Modal Solution Data", "Rectangular Plot", "Setup1 : Sweep", 
    	[
    		"Domain:="		, "Sweep"
    	], 
    	[
    		"Freq:="		, ["All"],
    	], 
    	[
    		"X Component:="		, "Freq",
    		"Y Component:="		, [resultName]
    	], [])

    oModule.ExportToFile(resultName, path+"/output/"+str(simID)+"/files/"+str(resultName)+str(name)+".csv")


#("Eigen Modes Plot 1",
def createEigenReport(proj,path,name,design,simID, resultName):
    oDesign = proj.SetActiveDesign(design)
    oModule = oDesign.GetModule("ReportSetup")
    oModule.CreateReport(resultName, "Eigenmode Parameters", "Rectangular Plot", "Setup1 : LastAdaptive", [], 
	[
		"px:="			, ["All"],
		"gWidth:="		, ["Nominal"],
		"Period:="		, ["Nominal"],
		"Segments:="		, ["Nominal"],
		"gap:="			, ["Nominal"],
		"h1_factor:="		, ["Nominal"],
		"w1_factor:="		, ["Nominal"],
		"offset_factor:="	, ["Nominal"],
		"h2_factor:="		, ["Nominal"],
		"sep_factor:="		, ["Nominal"],
		"w2_factor:="		, ["Nominal"],
		"SweepPML:="		, ["Nominal"],
		"SubsH:="		, ["Nominal"],
		"py:="			, ["0deg"]
	], 
	[
		"X Component:="		, "px",
		"Y Component:="		, ["re(Mode(1))","re(Mode(2))","re(Mode(3))"]
	])
    oModule.ExportToFile(resultName, path+"/output/"+str(simID)+"/files/"+str(resultName)+str(name)+".csv")
